<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
 
/**
 * Custom PHP JS Tools class.
 *
 * A collection of needed function in Custom PHP JS plugin.
 * Functions called statically, Uses the class just as a holder for functions.
 *
 * @since 1.0.0
 *
 * @see CPJ_Tools
 */
if ( ! class_exists( 'CPJ_Tools' ) )
{
    class CPJ_Tools
    {
        /**
         * Remove (unset) array keys with empty value.
         *
         * Modify array and sub array to remove keys with empty values.
         * Keep int 0 (false) values.
         *
         * @since 1.0.0
         *
         * @param array     $array  Format of the date to return.
         * 
         * @return array    $array  After remove keys with empty values.
         */
        static function array_remove_empty_deep( $array )
        {
            foreach ( $array as $key => $value ) {
                if ( is_array( $value ) ) {
                    $array[ $key ] = self::array_remove_empty_deep( $value );
                } else {
                    if ( trim( $value ) == "" ) {
                        unset( $array[ $key ] );
                    }
                }
            }
            return $array;
        }
        
        /**
         * Filter array values with filter function.
         *
         * Modify array and sub array, filter values with custom
         * callback function unsing call_user_func function.
         *
         * @since 1.0.0
         *
         * @param string $callback    Function (filter) to apply on array values.
         * @param array  $array       Array to filter.
         * 
         * @return array    Formatted and filtered array.
         */
        static function array_map_recursive( $callback, $array )
        {
            foreach ( $array as $key => $value ) {
                if ( is_array( $array[ $key ] ) ) {
                    $array[ $key ] = self::array_map_recursive( $callback, $array[ $key ] );
                }
                else {
                    $array[ $key ] = call_user_func( $callback, $array[ $key ] );
                }
            }
            return $array;
        }
        
        /**
         * Sanitize textarea text using WordPress functions.
         *
         * Filter text using WordPress built in function sanitize_text_field
         * but keep newlines and tabs in text (usually removed by sanitize_text_field) 
         *
         * @since 1.0.0
         *
         * @param string    $text   Text to sanitize.
         * 
         * @return string   Formatted string $text.
         */
        static function sanitize_textarea( $text )
        {
            // keep lines from sanitize
            $lines = explode( "\n", (string) $text );
            // keep taps from sanitize
            foreach( $lines as $k => $line ) {
                $line = explode( "\t", (string) $line );
                $line = array_map( 'sanitize_text_field', (array) $line );
                $lines[$k] = implode( "\t", $line );
            }
            $text = implode( "\n", $lines );
            return $text;
        }
       
        /**
         * Convert given key string into a valid formated key.
         *
         * Format a given key to a valid and save to use php or WordPress key
         * using WordPress sanitize_key function after remove spaces and dashes.
         *
         * @since 1.0.0
         *
         * @param string    $key   Key String to format.
         * 
         * @return string   Formatted date string or Unix timestamp. False if $date is empty.
         */
        static function generateId( $key )
        {
            $key = str_replace( ' ', '_', $key );   // replace spaces before sanitize remove them.
            $key = sanitize_key( $key );            // wordpress sanitize key.
            $key = str_replace( '-', "_", $key );   // replace dashes to underscores.
            $key = str_replace( '__', "_", $key );  // replace duplicated underscores.
            $key = str_replace( '___', "_", $key ); // replace duplicated underscores.
            $key = str_replace( '____', "_", $key );// replace duplicated underscores.
            return $key;
        }
        
        /**
         * Convert given string id or key into a readable name.
         *
         * Formate an id or WordPress key into a beautiful readable name 
         *
         * @since 1.0.0
         *
         * @param string    $id String to format.
         * 
         * @return string   Readable Formatted name string.
         */
        static function id2title( $id )
        {
            $id = str_replace( '_', ' ', $id ); // replace underscores with space.
            $id = str_replace( '-', ' ', $id ); // replace dashes with space.
            $id = ucwords( $id );   // Capitalize first letter of each word
            return $id;
        }
        
        /**
         * Check if a given string is a json code.
         *
         * @since 1.0.0
         *
         * @param string    $string Format of the date to return.
         * 
         * @return bool     True if string $string is json, False if not.
         */
        static function is_json( $string )
        {
            //&& (json_last_error() == JSON_ERROR_NONE) 
            return is_string( $string ) && is_object( json_decode( $string ) ) ? true : false;
        }
        
        /**
         * Return default value in $value is not set.
         *
         * @since 1.0.0
         *
         * @param string $value     Value to check with isset php funtion.
         * @param string $default   Default value to return if $value is not set.
         * 
         * @return string   Given value or default one.
         */
        static function set_default( $value, $default )
        {
            return isset( $value )? $value : $default; 
        }
        
        /**
         * Get the boolean value of a variable.
         *
         * Same as php built in function boolval supported in php 5 >= 5.5.0
         *
         * @since 1.0.0
         *
         * @param string    $val    The scalar value being converted to a boolean.
         * 
         * @return bool     The boolean value of $val.
         */
        static function boolval( $val )
        {
            return (bool) $val;
        }
        
        /**
         * Hook a function to a specific filters action.
         *
         * @since 1.0.0
         *
         * @param string    $tags      Array of names of the existing Filter to Hook the $function_to_add argument to.
         * @param string    $function  The name of the function to be called when the custom Filter is applied.
         * 
         * @return bool     The function returns true if the attempted function hook succeeds or false if not.
         */
        static function add_filters( $tags, $function )
        {
            foreach ( $tags as $tag ) {
                add_filter( $tag, $function );
            }
        }
        
        /**
         * Check if element is empty string
         *
         * @since 1.0.0
         *
         * @param string    $element    Element to check if empty.
         * 
         * @return bool     True if empty. False if not.
         */
        static function is_element_empty( $element )
        {
            $element = trim( $element );
            return !empty( $element );
        }
        
        /**
         * Get array of registered post type
         *
         * If $opgroup is false, Returned array key is post type key, value is post type label name.
         * If $opgroup is true, Returned array key is opgroup label name, value is key value array 
         * of post types under this group.
         *
         * @since 1.0.0
         *
         * @param bool  $opgroup    Whether to return builtin post types in sub array suitable for select feild opgroup. Default false.
         * @param bool  $builtin    Whether to return also WordPress builtin post types. Default true.
         * @param array $break      Array of post types keys to avoid and remove from returned array.
         * 
         * @return array a key value array of registered post types.
         */
        static function post_type_array( $opgroup = false, $builtin = true, $break = array( 'revision', 'nav_menu_item', 'attachment' ) )
        {
            $zomax_post_types_custom = get_post_types( array( '_builtin' => false ), 'objects' );

            $options = array();
            if( $builtin ) {
                $zomax_post_types_builtin = get_post_types( array( '_builtin' => true ), 'objects' );
                $opGroupTitle = __( 'Post Types (Built-in)', 'zomax' );
                foreach( $zomax_post_types_builtin as $post_type => $pt ) {
                    if ( ! in_array( $post_type, $break ) ) {
                        if( $opgroup ) {
                            $options[ $opGroupTitle ][ $post_type ] = ucfirst( esc_html( __( $pt->label, 'zomax' ) ) );
                        }else {
                            $options[ $post_type ] = ucfirst( esc_html( __($pt->label, 'zomax' ) ) );
                        }
                    }
                } 
            }
            $opGroupTitle = __( 'Post Types (Custom)', 'zomax' );
            foreach ( $zomax_post_types_custom as $post_type => $pt ){
                    if ( in_array( $post_type, $break ) ){ break 1; }
                    if( $opgroup ){
                        $options[ $opGroupTitle ][ $post_type ] = ucfirst(esc_html( __( $pt->label, 'zomax' ) ) );
                    }else{
                        $options[ $post_type ] = ucfirst(esc_html( __( $pt->label, 'zomax' ) ) );
                    }
            } 
            return $options;
        }
        
        /**
         * Check if local file exist from URL.
         *
         * @since 1.0.0
         *
         * @param string    $src        Format of the date to return.
         * @param bool      $check_size Date string to convert.
         * 
         * @return bool True if exist. False if not.
         */
        static function is_file_exist( $src, $check_size = false )
        {
            $without_home_url = str_replace( home_url().'/', '', $src );
            $with_ABSPATH = ABSPATH . $without_home_url;
            
            if( $check_size && file_exists( $with_ABSPATH ) && filesize( $with_ABSPATH ) > 0 )
            {
                return true;
            }elseif( ! $check_size && file_exists( $with_ABSPATH ) )
            {
                return true;
            }else
            {
                return false;
            }
        }
        
        /**
         * Enqueue a script.
         *
         * Registers the script if $src provided (does NOT overwrite), and enqueues it.
         *
         * Using WordPress function 'wp_enqueue_script' after check rtl support.
         * 
         * @link //codex.wordpress.org/Function_Reference/wp_enqueue_script
         *
         * @since 1.0.0
         *
         * @param string      $handle    Name of the script.
         * @param string|bool $src       Path to the script from the root directory of WordPress. Example: '/js/myscript.js'.
         * @param array       $deps      An array of registered handles this script depends on. Default empty array.
         * @param string|bool $ver       Optional. String specifying the script version number, if it has one. This parameter
         *                               is used to ensure that the correct version is sent to the client regardless of caching,
         *                               and so should be included if a version number is available and makes sense for the script.
         * @param bool        $in_footer Optional. Whether to enqueue the script before </head> or before </body>.
         *                               Default 'false'. Accepts 'false' or 'true'.
         * @param bool        $error     Default false. Accepts false or true.
         */
        static function enqueue_script( $handle, $src = false, $deps = array(), $ver = false, $in_footer = false, $error = true )
        {
            if ( $src ) {
                wp_deregister_script( $handle );
                $s = explode( '.', $src );
                $extension = end( $s );
                $src_without_extension = str_replace( '.' . $extension, '', $src );
                if( is_rtl() && self::is_file_exist( $src_without_extension . '-rtl.' . $extension ) ) {
                    $src = $src_without_extension . '-rtl.' . $extension;
                    wp_enqueue_script( $handle, $src, $deps, $ver, $in_footer );
                }elseif( ! $error ) {
                    if( self::is_file_exist( $src, true ) )
                    {
                        wp_enqueue_script( $handle, $src, $deps, $ver, $in_footer );
                    }
                }else {
                    wp_enqueue_script( $handle, $src, $deps, $ver, $in_footer );
                }
            }else
            {
                wp_enqueue_script( $handle, $src, $deps, $ver, $in_footer );
            }
        }
        
        /**
         * Enqueue a CSS stylesheet.
         *
         * Using WordPress function 'wp_enqueue_style' after check rtl support.
         * 
         * @link //codex.wordpress.org/Function_Reference/wp_enqueue_style
         *
         * @since 1.0.0
         *
         * @param string      $handle Name of the stylesheet.
         * @param string|bool $src    Path to the stylesheet from the root directory of WordPress. Example: '/css/mystyle.css'.
         * @param array       $deps   An array of registered style handles this stylesheet depends on. Default empty array.
         * @param string|bool $ver    String specifying the stylesheet version number, if it has one. This parameter is used
         *                            to ensure that the correct version is sent to the client regardless of caching, and so
         *                            should be included if a version number is available and makes sense for the stylesheet.
         * @param string      $media  Optional. The media for which this stylesheet has been defined.
         *                            Default 'all'. Accepts 'all', 'aural', 'braille', 'handheld', 'projection', 'print',
         *                            'screen', 'tty', or 'tv'.
         */
        static function enqueue_style( $handle, $src = false, $deps = array(), $ver = false, $media = 'all', $error = true )
        {
            if ( $src ) {
                wp_deregister_style( $handle );
                $s = explode( '.', $src );
                $extension = end( $s );
                $src_without_extension = str_replace('.' . $extension, '', $src );
                if( is_rtl() && self::is_file_exist( $src_without_extension . '-rtl.' . $extension ) ) {
                    $src = $src_without_extension . '-rtl.' . $extension;
                    wp_enqueue_style( $handle, $src, $deps, $ver, $media );
                }elseif( ! $error ) {
                    if(self::is_file_exist( $src, true ) )
                    {
                        wp_enqueue_style( $handle, $src, $deps, $ver, $media );
                    }
                }else {
                    wp_enqueue_style( $handle, $src, $deps, $ver, $media );
                }
            }else
            {
                wp_enqueue_style( $handle, $src, $deps, $ver, $media );
            }
        }
        
        /**
         * Display a JAVASCRIPT code to create a screen notification.
         *
         * @since 1.0.0
         *
         * @param string $status    Default 'success'. Accepts 'success', 'warning', 'danger', 'info'.
         * @param string $title     Date string to convert.
         * @param bool   $msg       Default empty.
         * @param int    $expires   Timeout before notification disappear in millisecond.
         */
        static function notify( $status = "success", $title = null, $msg = "", $expires = "8000" )
        {
            if( is_null( $title ) ) {
                $title =  __( "Request Done!", "zomax" );
            }
            ?>
            <script>
            //<![CDATA[
                cpjNotify.notify("create", "<?php echo $status; ?>", {
                  title: "<?php echo $title; ?>",
                  msg: "<?php echo $msg; ?>"
                },{expires:<?php echo $expires; ?>});
            //]]>
            </script>
            <?php
        }
    }
}